package com.sky.utils;
import com.alibaba.fastjson.JSONObject;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.HttpEntity;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.ByteArrayEntity;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;
import java.io.IOException;
import java.net.URLConnection;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;

@Slf4j
@AllArgsConstructor
@Data
public class FileBrowserUtil {
    private String domain;
    private String username;
    private String password;

    /**
     * —— 第一步：登录拿 token ——
     * 调用 /api/login 接口，返回原始的 JWT token 字符串
     * curl -X POST "https://fshare.bitday.top/api/login" \
     *      -H "Content-Type: application/json" \
     *      -d '{
     *            "username":"admin",
     *            "password":"asdf14789"
     *          }'
     * 返回值： eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9…
     */

    public String login() throws IOException {
        String url = domain + "/api/login";
        // 创建 HttpClient 实例
        try (CloseableHttpClient httpClient = HttpClients.createDefault()) {
            // 构造 POST 请求
            HttpPost httpPost = new HttpPost(url);
            httpPost.setHeader("Content-Type", "application/json");

            // 构造 JSON body
            JSONObject json = new JSONObject();
            json.put("username", username);
            json.put("password", password);

            // 设置请求体
            StringEntity requestEntity = new StringEntity(json.toString(), StandardCharsets.UTF_8);
            requestEntity.setContentType("application/json");
            httpPost.setEntity(requestEntity);

            // 发送请求并处理响应
            try (CloseableHttpResponse response = httpClient.execute(httpPost)) {
                int statusCode = response.getStatusLine().getStatusCode();
                if (statusCode >= 200 && statusCode < 300) {
                    HttpEntity respEntity = response.getEntity();
                    String body = EntityUtils.toString(respEntity, StandardCharsets.UTF_8);
                    log.info("token:{}",body);
                    // 返回原始返回值（假设就是 JWT token 字符串）
                    return body;
                } else {
                    log.error("Login failed, HTTP status code: " + statusCode);
                    return "";
                }
            }
        }
    }

    /**
     * —— 第二步：上传文件 ——
     +     * curl -v -X POST \
     +     *   "$DOMAIN/api/resources/$REMOTE_PATH?override=true" \      // 服务器上相对路径
     +     *   -H "X-Auth: $TOKEN" \
     +     *   -H "Content-Type: image/jpeg" \                         // 根据文件类型替换
     +     *   --data-binary "@/path/to/local/photo.jpg"               // 以 raw body 方式上传
     */

    public String uploadAndGetUrl(byte[] fileBytes, String fileName) throws IOException {
        // 1. 登录拿 token
        String token = login();

        // 2. 确定远端存储路径和编码（保留斜杠）
        String remotePath = "store/" + fileName;
        String encodedPath = URLEncoder.encode(remotePath, StandardCharsets.UTF_8.toString())
                .replace("%2F", "/");

        // 3. 根据文件名猜 MIME 类型，fallback 到 application/octet-stream
        String mimeType = URLConnection.guessContentTypeFromName(fileName);
        if (mimeType == null) {
            mimeType = "application/octet-stream";
        }

        // 4. 构造上传 URL
        String uploadUrl = domain + "/api/resources/" + encodedPath + "?override=true";

        // 5. 执行 upload
        try (CloseableHttpClient client = HttpClients.createDefault()) {
            HttpPost post = new HttpPost(uploadUrl);
            post.setHeader("X-Auth", token);
            post.setEntity(new ByteArrayEntity(fileBytes, ContentType.create(mimeType)));
            try (CloseableHttpResponse resp = client.execute(post)) {
                int status = resp.getStatusLine().getStatusCode();
                String body = EntityUtils.toString(resp.getEntity(), StandardCharsets.UTF_8);
                if (status < 200 || status >= 300) {
                    log.error("文件上传失败: HTTP {}，{}", status, body);
                    return "";
                }
            }
        }

        // 6. 拼接 raw 下载链接并返回
        String downloadUrl = domain + "/api/raw/" + encodedPath;
        log.info("文件下载链接：{}", downloadUrl);
        return downloadUrl;
    }
}
